// Ionic Starter App

// angular.module is a global place for creating, registering and retrieving Angular modules
// 'starter' is the name of this angular module example (also set in a <body> attribute in index.html)
// the 2nd parameter is an array of 'requires'
// 'starter.controllers' is found in controllers.js
angular.module('Court', [
    'ionic',
    'ngCordova',
    'ui.bootstrap.datetimepicker'
])

    .run(function ($rootScope, $ionicPlatform, $state, $localstorage, Factory, $ionicLoading, $ionicPopup) {
        $ionicPlatform.ready(function () {
            // Hide the accessory bar by default (remove this to show the accessory bar above the keyboard
            // for form inputs)
            if (window.cordova && window.cordova.plugins.Keyboard) {
                cordova.plugins.Keyboard.hideKeyboardAccessoryBar(true);
            }

            if (window.StatusBar) {
                // org.apache.cordova.statusbar required
                StatusBar.styleDefault();
            }

            if (window.Connection) {
                if (navigator.connection.type == Connection.NONE) {
                    $state.go('connection')
                }
            }

            $rootScope.$digest();
        });
        $rootScope.models = Factory.models;
        $rootScope.authData = $localstorage.getObject('auth');
        $rootScope.global_section = $localstorage.getObject('global_section');


        if (!$rootScope.global_section.is_old) {
            $localstorage.setObject('global_section', {is_old: false});
        }

        $rootScope.changeGlobalSection = function (global_section) {
            console.log('global_section:', global_section)
            $ionicLoading.show({
                template: 'Уншиж байна...'
            });
            $localstorage.setObject('global_section', {is_old: global_section});

            Factory.init(function (res) {
                $ionicLoading.hide();
            })

        };

        $rootScope.selectModel = function (model, type) {
            if (model && type) {
                $rootScope.model_section = model;
                $rootScope.section_type = type;
                $state.go('app.tabs.' + model + type)
            } else {
                $state.go('landing')
            }

        }

        $rootScope.selectZarToim = function (type) {
            if (type) {
                $rootScope.zartoim_type = type;

                if (type == 'zar') {
                    $rootScope.ztTitle = 'Шүүх хуралдааны зар';
                } else {
                    $rootScope.ztTitle = 'Шүүх хуралдааны тойм';
                }
                $state.go('app.zartoim')
            } else {
                $state.go('landing')
            }

        }

    })

    .config(function ($stateProvider, $urlRouterProvider) {
        $stateProvider

            .state('app', {
                url: "/app",
                abstract: true,
                templateUrl: "templates/root.html",
                controller: 'RootCtrl'
            })

            .state('app.tabs', {
                url: "/tab",
                abstract: true,
                views: {
                    'rootContent': {
                        templateUrl: "templates/tabs.html"
                    }
                }
            })

            .state('landing', {
                url: "/landing",
                templateUrl: "templates/landing.html",
                controller: 'LandingCtrl'
            })
            .state('connection', {
                url: "/connection",
                templateUrl: "templates/connection.html",
                controller: 'ConnectionCtrl'
            })

//=============================================================================================
            .state('app.tabs.irgenanhan', {
                url: "/irgen/anhan",
                views: {
                    'anhan-tab': {
                        templateUrl: "templates/citizen/list.html",
                        controller: 'CitizenCtrl'
                    }
                }
            })
            .state('app.tabs.irgendavah', {
                url: "/irgen/davah",
                views: {
                    'davah-tab': {
                        templateUrl: "templates/citizen/list.html",
                        controller: 'CitizenCtrl'
                    }
                }
            })
            .state('app.tabs.irgenhyanalt', {
                url: "/irgen/hyanalt",
                views: {
                    'hyanalt-tab': {
                        templateUrl: "templates/citizen/list.html",
                        controller: 'CitizenCtrl'
                    }
                }
            })
//=============================================================================================

            .state('app.tabs.eruuanhan', {
                url: "/eruu/anhan",
                views: {
                    'anhan-tab': {
                        templateUrl: "templates/criminal/list.html",
                        controller: 'CriminalCtrl'
                    }
                }
            })
            .state('app.tabs.eruudavah', {
                url: "/eruu/davah",
                views: {
                    'davah-tab': {
                        templateUrl: "templates/criminal/list.html",
                        controller: 'CriminalCtrl'
                    }
                }
            })
            .state('app.tabs.eruuhyanalt', {
                url: "/eruu/hyanalt",
                views: {
                    'hyanalt-tab': {
                        templateUrl: "templates/criminal/list.html",
                        controller: 'CriminalCtrl'
                    }
                }
            })

//=============================================================================================

            .state('app.tabs.zahirgaaanhan', {
                url: "/zahirgaa/anhan",
                views: {
                    'anhan-tab': {
                        templateUrl: "templates/zahirgaa/list.html",
                        controller: 'ZahirgaaCtrl'
                    }
                }
            })
            .state('app.tabs.zahirgaadavah', {
                url: "/zahirgaa/davah",
                views: {
                    'davah-tab': {
                        templateUrl: "templates/zahirgaa/list.html",
                        controller: 'ZahirgaaCtrl'
                    }
                }
            })
            .state('app.tabs.zahirgaahyanalt', {
                url: "/zahirgaa/hyanalt",
                views: {
                    'hyanalt-tab': {
                        templateUrl: "templates/zahirgaa/list.html",
                        controller: 'ZahirgaaCtrl'
                    }
                }
            })
//=============================================================================================
            .state('app.zartoim', {
                url: "/zartoim",
                views: {
                    'rootContent': {
                        templateUrl: "templates/zartoim/list.html",
                        controller: 'ZarToimCtrl'
                    }
                }
            })

//=============================================================================================
            .state('app.shuuh', {
                url: "/shuuh/map",
                views: {
                    'rootContent': {
                        templateUrl: "templates/shuuh/map.html",
                        controller: 'MapCtrl'
                    }
                }
            })
//=============================================================================================
            .state('app.lavlagaa', {
                url: "/lavlagaa",
                views: {
                    'rootContent': {
                        templateUrl: "templates/lavlagaa/lavlagaa.html",
                        controller: 'LavlagaaCtrl'
                    }
                }
            })
//=============================================================================================

        // if none of the above states are matched, use this as the fallback
        $urlRouterProvider.otherwise('/landing');
    });

